@@ -3,8 +3,8 @@
 ;; Copyright © 2020, by Sebastian Meisel
 
 ;; Author: Sebastian Meisel <sebastian.meisel@gmail.com>
-;; Package-Version: 1.1
-;; Package-Revision: aef916217111
+;; Package-Version: 20250922.1323
+;; Package-Revision: c5a6127ad831
 ;; Created:  November 12, 2023
 ;; Keywords: unix
 ;; Homepage: https://github.com/SebastianMeisel/journalctl-mode
@@ -43,6 +43,10 @@
 
 ;; Put journalctl-mode.el in your load-path and add   ( require 'journalctl-mode)  to your .emacs file.
 
+;; It might be a good idea to define a global key to journalctl with:
+;; (global-set-key (kbd "C-c t") 'journalctl).
+
+
 ;;; Code:
 
 (require 'array)
@@ -85,18 +89,17 @@
   :group 'journalctl
   :type 'string)
 
-(defcustom journalctl-follow-freq
-  0.1
-  "Frequency in seconds for follow simulation."
-  :group 'journalctl
-  :type 'float)
-
 (defcustom journalctl-follow-lines
-  "30"
+  "240"
   "Number of lines for follow simulation."
   :group 'journalctl
   :type 'string)
 
+(defcustom journalctl-default-options 
+  '("--lines=240")
+  "List of default options on start."
+  :group 'journalctl
+  :type 'list)
 
 ;;; faces
 (defface journalctl-error-face
@@ -168,19 +171,19 @@
 
 (defvar journalctl-output-list
   '("short-full"
-        "short-iso"
-        "short-iso-precise"
-        "short-precise"
-        "short-monotonic"
-        "short-unix"
-        "verbose"
-        "export"
-        "json"
-        "json-pretty"
-        "json-sse"
-        "json-seq"
-        "cat"
-        "with-unit")
+    "short-iso"
+    "short-iso-precise"
+    "short-precise"
+    "short-monotonic"
+    "short-unix"
+    "verbose"
+    "export"
+    "json"
+    "json-pretty"
+    "json-sse"
+    "json-seq"
+    "cat"
+    "with-unit")
   "Options for the --output parameter.
 It controls the formatting of the journal entries that are shown.")
 
@@ -252,30 +255,30 @@ It controls the formatting of the journal entries that are shown.")
       "0G")))
 
 (transient-define-infix journalctl-transient:--lines ()
-		       :description "Limit number of events."
-		       :class 'transient-option
-		       :shortarg "n"
-		       :argument "--lines="
-		       )
+  :description "Limit number of events."
+  :class 'transient-option
+  :shortarg "n"
+  :argument "--lines="
+  )
 
 (transient-define-infix journalctl-transient:--grep ()
-		       :description "Grep for a pattern."
-		       :class 'transient-option
-		       :shortarg "g"
-		       :argument "--grep="
-		       )
+  :description "Grep for a pattern."
+  :class 'transient-option
+  :shortarg "g"
+  :argument "--grep="
+  )
 
 
 (transient-define-infix journalctl-transient:--outputs ()
-		       :description "Controls the formatting."
-		       :class 'transient-option
-		       :shortarg "o o"
-		       :argument "--output="
-		       :choices journalctl-output-list
-		       )
+  :description "Controls the formatting."
+  :class 'transient-option
+  :shortarg "o o"
+  :argument "--output="
+  :choices journalctl-output-list
+  )
 
 (transient-define-infix journalctl-transient:--field ()
-  :description "Print all values for a specific field."
+  :description "Values for a specific field."
   :class 'transient-option
   :shortarg "o f"
   :argument "--field="
@@ -283,148 +286,148 @@ It controls the formatting of the journal entries that are shown.")
   )
 
 (transient-define-infix journalctl-transient:--boot ()
-		       :description "Show boot log."
-		       :class 'transient-option
-		       :shortarg "c b"
-		       :argument "--boot="
-		       :choices journalctl-boot-list
-		       )
+  :description "Boot log."
+  :class 'transient-option
+  :shortarg "c b"
+  :argument "--boot="
+  :choices journalctl-boot-list
+  )
 
 (transient-define-infix journalctl-transient:--unit ()
-		       :description "Show log for specific systemd unit."
-		       :class 'transient-option
-		       :shortarg "c u"
-		       :argument "--unit="
-		       :choices journalctl-unit-list
-		       )
+  :description "Specific systemd unit."
+  :class 'transient-option
+  :shortarg "c u"
+  :argument "--unit="
+  :choices journalctl-unit-list
+  )
 
 (transient-define-infix journalctl-transient:--user-unit ()
-		       :description "Show log for specific systemd user unit."
-		       :class 'transient-option
-		       :shortarg "c U"
-		       :argument "--user-unit="
-		       :choices journalctl-user-unit-list
-		       )
+  :description "Specific systemd user unit."
+  :class 'transient-option
+  :shortarg "c U"
+  :argument "--user-unit="
+  :choices journalctl-user-unit-list
+  )
 
 (transient-define-infix journalctl-transient:--priority ()
-		       :description "Filter output by message priorities (ranges)."
-		       :class 'transient-option
-		       :shortarg "c p"
-		       :argument "--priority="
-		       :choices journalctl-priority-list
-		       )
+  :description "Filter by priorities (ranges)."
+  :class 'transient-option
+  :shortarg "c p"
+  :argument "--priority="
+  :choices journalctl-priority-list
+  )
 
 (transient-define-infix journalctl-transient:--identifier ()
-		       :description "Show messages for specified syslog identifier."
-		       :class 'transient-option
-		       :shortarg "c t"
-		       :argument "--identifier="
-		       )
+  :description "Specified syslog identifier."
+  :class 'transient-option
+  :shortarg "c t"
+  :argument "--identifier="
+  )
 
 (transient-define-infix journalctl-transient:--facility ()
-		       :description "Filter output by syslog facility."
-		       :class 'transient-option
-		       :shortarg "c f"
-		       :argument "--facility="
-		       :choices journalctl-facility-list
-		       )
+  :description "Specific syslog facility."
+  :class 'transient-option
+  :shortarg "c f"
+  :argument "--facility="
+  :choices journalctl-facility-list
+  )
 
 (transient-define-infix journalctl-transient:--since ()
-		       :description  "Start showing entries on or newer than the specified date."
-		       :class 'transient-option
-		       :shortarg "S"
-		       :argument "--since="
-		       :prompt "Since (2012-10-30 18:17:16|yesterday|today|now|+|-):"
-		       )
+  :description  "Entries on or newer than date."
+  :class 'transient-option
+  :shortarg "S"
+  :argument "--since="
+  :prompt "Since (2012-10-30 18:17:16|yesterday|today|now|+|-):"
+  )
 
 (transient-define-infix journalctl-transient:--until ()
-		       :description  "Start showing entries on or older than the specified date."
-		       :class 'transient-option
-		       :shortarg "U"
-		       :argument "--until="
-		       :prompt "Until (2012-10-30 18:17:16|yesterday|today|now|+|-):"
-		       )
+  :description  "Entries on or older than date."
+  :class 'transient-option
+  :shortarg "U"
+  :argument "--until="
+  :prompt "Until (2012-10-30 18:17:16|yesterday|today|now|+|-):"
+  )
 
 (transient-define-infix journalctl-transient:--machine ()
-		       :description  "Show messages from a running, local container."
-		       :class 'transient-option
-		       :shortarg "s m"
-		       :argument "--machine="
-		       )
+  :description  "Messages from running, local container."
+  :class 'transient-option
+  :shortarg "s m"
+  :argument "--machine="
+  )
 
 (transient-define-infix journalctl-transient:--directory ()
-		       :description  "Directory on which journalctl shall operate."
-		       :class 'transient-option
-		       :shortarg "s d"
-		       :argument "--directory="
-		       )
+  :description  "Directory to operate on."
+  :class 'transient-option
+  :shortarg "s d"
+  :argument "--directory="
+  )
 
 (transient-define-infix journalctl-transient:--file ()
-		       :description  "File glob on which journalctl shall operate."
-		       :class 'transient-option
-		       :shortarg "s f"
-		       :argument "--file="
-		       )
+  :description  "File glob to operate on."
+  :class 'transient-option
+  :shortarg "s f"
+  :argument "--file="
+  )
 
 (transient-define-infix journalctl-transient:--root ()
-		       :description  "Directory root on which journalctl shall operate."
-		       :class 'transient-option
-		       :shortarg "s r"
-		       :argument "--root="
-		       )
+  :description  "Directory root to operate on."
+  :class 'transient-option
+  :shortarg "s r"
+  :argument "--root="
+  )
 
 (transient-define-infix journalctl-transient:--image ()
-		       :description  "Image file on which journalctl shall operate."
-		       :class 'transient-option
-		       :shortarg "s i"
-		       :argument "--image="
-		       )
+  :description  "Image file to operate on."
+  :class 'transient-option
+  :shortarg "s i"
+  :argument "--image="
+  )
 
 (transient-define-infix journalctl-transient:--namespace ()
-		       :description  "namespace on which journalctl shall operate."
-		       :class 'transient-option
-		       :shortarg "s i"
-		       :argument "--namespace="
-		       )
+  :description  "Namespace to operate on."
+  :class 'transient-option
+  :shortarg "s i"
+  :argument "--namespace="
+  )
 
 (transient-define-prefix journalctl-transient ()
   "Transient for journalctl."
+  :value journalctl-default-options
   [["Output"
-   ("o a" " Show all fields in full."
-   ("o l" "Ellipsize fields when they do not fit." "--no-full")
-    "--all")
-   ("o m" "Show entries interleaved from all available journals." "--merge")
-   (journalctl-transient:--field)
-   ("o q" "Suppresses all informational messages." "--quiet")
-   ("o u" "Express time in Coordinated Universal Time (UTC)." "--utc")
-   (journalctl-transient:--outputs)
-   ]
-  ["Sources"
-   (journalctl-transient:--machine)
-   (journalctl-transient:--directory)
-   (journalctl-transient:--file)
-   (journalctl-transient:--root)
-   (journalctl-transient:--image)
-   (journalctl-transient:--namespace)
-   ]]
+    ("o a" "All fields in full." "--all")
+    ("o l" "Ellipsize fields when they do not fit." "--no-full")
+    ("o m" "Entries interleaved from all available journals." "--merge")
+    (journalctl-transient:--field)
+    ("o q" "Suppresses all informational messages." "--quiet")
+    ("o u" "Express time in UTC." "--utc")
+    (journalctl-transient:--outputs)
+    ]
+   ["Sources"
+    (journalctl-transient:--machine)
+    (journalctl-transient:--directory)
+    (journalctl-transient:--file)
+    (journalctl-transient:--root)
+    (journalctl-transient:--image)
+    (journalctl-transient:--namespace)
+    ]]
   [["Constraint"
-   ("C k" "Show only kernel messages. This implies -b." "--dmesg")
-   ("C s" "Show only system and kernel messages." "--system")
-   ("C u" "Show only system and kernel messages." "--user")
-   (journalctl-transient:--boot)
-   (journalctl-transient:--identifier)
-   (journalctl-transient:--unit)
-   (journalctl-transient:--user-unit)
-   (journalctl-transient:--facility)
-   ]
-  ["Filters"
-   (journalctl-transient:--since)
-   (journalctl-transient:--until)
-   ("r" "Reverse output (newest entries first)." "--reverse")
-   ("x" "Augment log lines with explanations." "--catalog")
-   (journalctl-transient:--lines)
-   (journalctl-transient:--grep)
-   ]]
+    ("C k" "Kernel messages only. This implies -b." "--dmesg")
+    ("C s" "System and kernel messages only." "--system")
+    ("C u" "User  messages only." "--user")
+    (journalctl-transient:--boot)
+    (journalctl-transient:--identifier)
+    (journalctl-transient:--unit)
+    (journalctl-transient:--user-unit)
+    (journalctl-transient:--facility)
+    ]
+   ["Filters"
+    (journalctl-transient:--since)
+    (journalctl-transient:--until)
+    ("r" "Reverse output (newest entries first)." "--reverse")
+    ("x" "Augment log lines with explanations." "--catalog")
+    (journalctl-transient:--lines)
+    (journalctl-transient:--grep)
+    ]]
   ["Aufruf"
    (journalctl-standard-suffix)
    (journalctl-close-menu-suffix)
@@ -435,7 +438,7 @@ It controls the formatting of the journal entries that are shown.")
 (transient-define-suffix journalctl-standard-suffix ()
   :transient t
   :key "SPC"
-  :description "Run journalctl with transient arguments on current chunk. KEEP menu."
+  :description "Run journalctl - KEEP menu."
   (interactive)
   (let* ((args (transient-args (oref transient-current-prefix command))))
     (journalctl--run args journalctl-current-chunk)))
@@ -443,38 +446,28 @@ It controls the formatting of the journal entries that are shown.")
 (transient-define-suffix journalctl-follow-suffix ()
   :transient nil
   :key "f"
-  :description "Run journalctl with transient arguments on current chunk in follow mode."
+  :description "Run journalctl in FOLLOW mode."
   (interactive)
   (let ((args (transient-args (oref transient-current-prefix command)))
-	(follow-args (concat "--lines=" journalctl-follow-lines " -e")))
+	(follow-args (concat "--lines=" journalctl-follow-lines " -f")))
     (cl-pushnew follow-args args)
-    (setq journalctl-follow-timer
-	  (run-with-timer journalctl-follow-freq journalctl-follow-freq
-			  'journalctl--run args journalctl-current-chunk))))
+    ;;    (setq journalctl-follow-timer
+    ;;	  (run-with-timer journalctl-follow-freq journalctl-follow-freq
+    (journalctl--follow args)))
 
 (transient-define-suffix journalctl-close-menu-suffix ()
   :transient nil
   :key "RET"
-  :description "Run journalctl with transient arguments on current chunk. CLOSE menu."
+  :description "Run journalctl - CLOSE menu."
   (interactive)
   (let* ((args (transient-args (oref transient-current-prefix command))))
     (journalctl--run args journalctl-current-chunk))
-    (setq buffer-read-only t))
-
-;; (defun journalctl-opts-to-alist (opt-list)
-;;   "Convert the string of command line parameters into a alist (PARAMETER . OPTION)."
-;;   (mapcar 'car
-;;	  (mapcar 'cl--plist-to-alist
-;;		  (mapcar
-;;		   (lambda
-;;		     (element)
-;;		     (string-split element "=" nil))
-;;		   opt-list))))
+  (setq buffer-read-only t))
 
 (defun journalctl ()
   "Run journalctl and open transient menu."
   (interactive)
-  (journalctl--run '("--lines=250"))
+  (journalctl--run journalctl-default-options)
   (journalctl-transient))
 
 (defun journalctl--run (transient-opts &optional chunk)
@@ -486,16 +479,54 @@ It controls the formatting of the journal entries that are shown.")
          (first-line (+ 1 (* this-chunk journalctl-chunk-size)))
          (last-line (+ first-line journalctl-chunk-size))
 	 (command `("bash"
-		      "-c"
-		      ,(concat "journalctl "
-			      opts
-			      " | sed -ne '"
-			      (int-to-string first-line)
-			      ","
-			      (int-to-string last-line)
-			      "p'"))))
+		    "-c"
+		    ,(concat "journalctl "
+			     opts
+			     " | sed -ne '"
+			     (int-to-string first-line)
+			     ","
+			     (int-to-string last-line)
+			     "p'"))))
     (with-current-buffer (get-buffer-create "*journalctl*")
       (switch-to-buffer "*journalctl*")
+      (let ((inhibit-read-only t))
+        (setq buffer-read-only nil)
+        (erase-buffer))
+      (setq journalctl-process
+            (make-process
+             :name "journalctl"
+             :buffer "*journalctl*"
+             :command command
+             :stderr (get-buffer-create "*journalctl-errors*")
+             :file-handler t
+             :sentinel #'ignore
+             :filter (lambda (proc string)
+                       (when (buffer-live-p (process-buffer proc))
+                         (with-current-buffer (process-buffer proc)
+                           (let ((inhibit-read-only t))
+                             (setq buffer-read-only nil)
+                             (let ((moving (= (point) (process-mark proc))))
+                               (save-excursion
+                                 (goto-char (process-mark proc))
+                                 (insert string)
+                                 (set-marker (process-mark proc) (point)))
+                               (if moving (goto-char (process-mark proc)))
+                               (goto-char (point-min))))
+                           (journalctl-mode)
+                           (setq buffer-read-only t)))))))))
+
+(defun journalctl--follow (transient-opts)
+  "Run journalctl with given TRANSIENT-OPTS and follow the output."
+  (interactive (list (transient-args 'journalctl-transient)))
+  (setq journalctl-current-opts transient-opts)
+  (let* ((opts (mapconcat 'identity transient-opts " "))
+	 (command `("bash"
+		    "-c"
+		    ,(concat "journalctl "
+			     opts))))
+    (with-current-buffer (get-buffer-create "*journalctl*")
+      (switch-to-buffer "*journalctl*")
+      (journalctl-mode)
       (setq buffer-read-only nil)
       (erase-buffer)
       (setq journalctl-process
@@ -516,8 +547,11 @@ It controls the formatting of the journal entries that are shown.")
                                (insert string)
                                (set-marker (process-mark proc) (point)))
 			     (if moving (goto-char (process-mark proc)))
-			     (goto-char (point-min)))
-			     (journalctl-mode)))))))))
+			     (if (> (array-current-line)
+				    journalctl-chunk-size)
+				 (save-excursion
+				   (goto-char (point-min))
+				   (kill-line (/ journalctl-chunk-size 10)))))))))))))
 
 ;;;;;; Moving and Chunks
 
@@ -539,8 +573,8 @@ It controls the formatting of the journal entries that are shown.")
   (interactive)
   (let ((chunk (if (>= journalctl-current-chunk 1)
 		   (- journalctl-current-chunk 1) 0)))
-	(setq journalctl-current-chunk chunk)
-	(journalctl--run journalctl-current-opts chunk)))
+    (setq journalctl-current-chunk chunk)
+    (journalctl--run journalctl-current-opts chunk)))
 
 (defun journalctl-scroll-down ()
   "Scroll up journalctl output.
@@ -579,26 +613,26 @@ Move to next chunk when top of frame is reached."
 
 
 (defvar journalctl-font-lock-keywords
-      (let* (
-            ;; generate regex string for each category of keywords
-	     (error-keywords-regexp
-	      (regexp-opt journalctl-error-keywords 'words))
-	     (warn-keywords-regexp
-	      (regexp-opt journalctl-warn-keywords 'words))
-	     (starting-keywords-regexp
-	      (regexp-opt journalctl-starting-keywords 'words))
-	     (finished-keywords-regexp
-	      (regexp-opt journalctl-finished-keywords 'words)))
-        `((,warn-keywords-regexp . 'journalctl-warning-face)
-          (,error-keywords-regexp . 'journalctl-error-face)
-          (,starting-keywords-regexp . 'journalctl-starting-face)
-          (,finished-keywords-regexp . 'journalctl-finished-face)
-	  ("^\\([A-Z][a-z]+ [0-9]+ [0-9:]+\\)" . (1 'journalctl-timestamp-face))
-	  ("^\\([A-Z][a-z]+ [0-9]+ [0-9:]+\\) \\([-a-zA-Z]+\\)" . (2 'journalctl-host-face))
-	  ("^\\([A-Z][a-z]+ [0-9]+ [0-9:]+\\) \\([-a-zA-Z]+\\) \\(.*?:\\)" . (3 'journalctl-process-face))
-          ;; note: order above matters, because once colored, that part won't change.
-          ;; in general, put longer words first
-          )))
+  (let* (
+         ;; generate regex string for each category of keywords
+	 (error-keywords-regexp
+	  (regexp-opt journalctl-error-keywords 'words))
+	 (warn-keywords-regexp
+	  (regexp-opt journalctl-warn-keywords 'words))
+	 (starting-keywords-regexp
+	  (regexp-opt journalctl-starting-keywords 'words))
+	 (finished-keywords-regexp
+	  (regexp-opt journalctl-finished-keywords 'words)))
+    `((,warn-keywords-regexp . 'journalctl-warning-face)
+      (,error-keywords-regexp . 'journalctl-error-face)
+      (,starting-keywords-regexp . 'journalctl-starting-face)
+      (,finished-keywords-regexp . 'journalctl-finished-face)
+      ("^\\([A-Z][a-z]+ [0-9]+ [0-9:]+\\)" . (1 'journalctl-timestamp-face))
+      ("^\\([A-Z][a-z]+ [0-9]+ [0-9:]+\\) \\([-a-zA-Z]+\\)" . (2 'journalctl-host-face))
+      ("^\\([A-Z][a-z]+ [0-9]+ [0-9:]+\\) \\([-a-zA-Z]+\\) \\(.*?:\\)" . (3 'journalctl-process-face))
+      ;; note: order above matters, because once colored, that part won't change.
+      ;; in general, put longer words first
+      )))
 
 ;; keymap
 (defvar journalctl-mode-map
@@ -619,6 +653,12 @@ Move to next chunk when top of frame is reached."
 (defvar font-lock-defaults nil "Defaults for Font Lock mode specified by the major mode.")
 
 ;;;###autoload
+(defun journalctl ()
+  "Run journalctl and open transient menu."
+  (interactive)
+  (journalctl--run '("--lines=250"))
+  (journalctl-transient))
+
 (define-derived-mode journalctl-mode fundamental-mode "journalctl"
   "Major mode for viewing journalctl output."
   (setq mode-line-process
